--! @file
--
--! @brief Address decode logic for ipbus fabric. Given an address it returns which slave is addressed.
--
--! @details
--! This file has been AUTOGENERATED from the address table - do not hand edit
--!
--! We assume the synthesis tool is clever enough to recognise 
--! exclusive conditions in the if statement.
--
--! @author Dave Newbold
--! @date February 2011

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
use work.ipbus.all;

package ipbus_addr_decode is

  function ipbus_addr_sel(signal addr : in std_logic_vector(31 downto 0)) return integer;

end ipbus_addr_decode;

package body ipbus_addr_decode is

  function ipbus_addr_sel(signal addr : in std_logic_vector(31 downto 0)) return integer is
    variable sel : integer;
  begin
		if    std_match(addr, "-----------------000--000----000") then
			sel := 0; -- firmwareid / base 00000000 / mask 00000000
		elsif std_match(addr, "-----------------000--000----001") then
			sel := 1; -- gpio / base 00000001 / mask 00000000
		elsif std_match(addr, "-----------------000--000----010") then
			sel := 2; -- select / base 00000002 / mask 00000000
		elsif std_match(addr, "-----------------000--000----011") then
			sel := 3; -- mask / base 00000003 / mask 00000000
                elsif std_match(addr, "-----------------000--000----100") then
                        sel := 4; -- reset / base 00000004 / mask 00000000
		elsif std_match(addr, "-----------------000--001-------") then
			sel := 5; -- scshiftreg / base 00000080 / mask 0000007f
		elsif std_match(addr, "-----------------000--010-------") then
			sel := 6; -- rshiftreg / base 00000100 / mask 0000007f                        

		elsif std_match(addr, "-----------------111--0---------") then
			sel := 7; -- triggerctrl / base 00007000 / mask 00000fff
                elsif std_match(addr, "-----------------111--1---------") then
                        sel := 8; -- triggerdata / base 00007200 / mask 00000fff
                        
		elsif std_match(addr, "-----------------001--1---------") then                  
			sel := 10; -- adcctrl0 / base 00001200 / mask 00000fff
                elsif std_match(addr, "-----------------001--0---------") then                  
			sel := 11; -- adcdata0 / base 00001000 / mask 00000fff
                        
		elsif std_match(addr, "-----------------010--1---------") then
			sel := 12; -- adcctrl1 / base 00002200 / mask 00000fff
                elsif std_match(addr, "-----------------010--0---------") then
			sel := 13; -- adcdata1 / base 00002000 / mask 00000fff

		elsif std_match(addr, "-----------------011--1---------") then
			sel := 14; -- adcctrl2 / base 00003200 / mask 00000fff
                elsif std_match(addr, "-----------------011--0---------") then
			sel := 15; -- adcdata2 / base 00003000 / mask 00000fff
                        
		elsif std_match(addr, "-----------------100--1---------") then
			sel := 16; -- adcctrl3 / base 00004200 / mask 00000fff
                elsif std_match(addr, "-----------------100--0---------") then
			sel := 17; -- adcdata3 / base 00004000 / mask 00000fff
                        
		elsif std_match(addr, "-----------------101--1---------") then
			sel := 18; -- adcctrl4 / base 00005200 / mask 00000fff
                elsif std_match(addr, "-----------------101--0---------") then
			sel := 19; -- adcdata4 / base 00005000 / mask 00000fff
                        
		elsif std_match(addr, "-----------------110------------") then
			sel := 9; -- hostemac / base 00006000 / mask 00000fff
		else
			sel := 99;
		end if;
		return sel;
	end ipbus_addr_sel;
 
end ipbus_addr_decode;
